/* stream.h -- generic stream data type */

/* generic stream header
 * (the representation of every stream starts with this header) */
typedef struct generic_stream_struct {
	void		(*close)();
	int			(*getchr)();
	boolean		(*getline)();
	ulong		(*getpos)();
	void		(*putchr)();
	void		(*putstring)();
	void		(*setpos)();
} StreamProcs, *StreamPtr;

/*
 * StreamPtr	your_Create(...your args...)
 *
 * your_Create is supplied by a particular stream implementation.  for example,
 * a memory read stream is created by calling memstream_CreateRead(...).
 * once created, a stream responds to a subset of the following generic
 * stream operations.  (naturally, input streams don't need to respond to
 * output operations and vice versa):
 *
 * void		stream_Close(StreamPtr stream);
 * int		stream_GetChar(StreamPtr stream);
 * boolean	stream_GetLine(StreamPtr stream, char *line, ushort lineSize);
 * ulong	stream_GetPos(StreamPtr stream);
 * void		stream_PutChar(StreamPtr stream, char);
 * void		stream_PutString(StreamPtr stream, char *s);
 * void		stream_SetPos(StreamPtr stream, ulong newPos);
 */

#define stream_Close(s)			((void) (*(s)->close)(s))
#define stream_GetChar(s)		((int) (*(s)->getchr)(s))
#define stream_GetLine(s,l,z)	((boolean) (*(s)->getline)((s),(l),(z)))
#define stream_GetPos(s)		((ulong) (*(s)->getpos)(s))
#define stream_PutChar(s,c)		((void) (*(s)->putchr)((s),(char)(c)))
#define stream_PutString(s,str)	((void) (*(s)->putstring)((s),(str)))
#define stream_SetPos(s,p)		((void) (*(s)->setpos)((s),(p)))
