/* dxget.c -- program to get Yamaha exclusive messages and them write to file */

/*****************************************************************************
*	    Change Log
*  Date	    | Change
*-----------+-----------------------------------------------------------------
* 13-Jun-86 | Created Change Log
*  2-Oct-86 | JMaloney: Converted to Macintosh
*****************************************************************************/

#include "switches.h"

#include <StdIO.h>
#include "cext.h"
#include "cmdline.h"
#include "excldesc.h"
#include "midicode.h"
#include "mpu.h"
#include "userio.h"

#define BUFF_SIZE 17000
byte midibuff[BUFF_SIZE];

/* shared with macmidi.c or mpu.c: */
extern int xbufftail;
extern int xbuffmask;
extern byte *xbuff;
extern byte *buff;

/****************************************************************************
*
*	command line switches and options
*
****************************************************************************/

#define nswitches 8
private char *switches[nswitches] =
	{"-help", "-miditrace", "-m", "-trace", "-t", "-block", "-d", "-debug"};

#define noptions 1
private char *options[noptions] = {"-tune"};

/*****************************************************************************
*
*	routines local to this module
*
*****************************************************************************/

void		instruct(void);
boolean		output(FILE *, byte **, byte *);
void		_main(int, char *[]);

/****************************************************************************
*					instruct
* Effect:
*	prints instructions for program
****************************************************************************/

private void instruct()
{
printf("This program will let you save DX7 and TX816 voice programs.\n");
printf("To save DX7 programs, you must connect\n");
printf("	DX7 MIDI OUT to MPU 401 MIDI IN.\n");
printf("For TX816 programs, connect TX816 MIDI OUT to MPU 401 MIDI IN,\n");
printf("	and set the OUT SLOT (leftmost display) to the module\n");
printf("	from which you want data.\n");
printf("For DX7: push the orange FUNCTION key, then the green 8.  Push 8\n");
printf("	repeatedly until the display says SYS INFO AVAIL or\n");
printf("	SYS INFO UNAVAIL.  If the display says SYS INFO UNAVAIL,\n");
printf("	push YES to change the display to SYS INFO AVAIL.\n");
printf("	Now, select a voice with the INTERNAL or CARTRIDGE button\n");
printf("	followed by a (green) number.\n");
printf("For ALL DX7 voices: after getting the SYS INFO AVAIL display,\n");
printf("	push 8 again to get MIDI TRANSMIT ?.  Push YES.\n");
printf("For TX816: Hold down SELECT until you see UT in the display.\n");
printf("	Let up on SELECT and then push it once more to get the\n");
printf("	flashing dU.  Push YES to dump all data from this module.\n");
printf("	Hold down SELECT until you see digits in the display.\n");
printf("After data is dumped, dxget will tell you what it got and write\n");
printf("	data to a file.	 Use dxput to copy a file to a dx module.\n");
}

/****************************************************************************
*					main
* Inputs:
*	int argc: command line argument count
*	char *argv[]: command line argument array
* Effect:
*	main program -- prompt user for information and get the data
****************************************************************************/

void _main(argc, argv)
	int argc;
	char *argv[];
{
	char filename[100];		/* the output file name */
	FILE *fp;				/* the output file */
	boolean done = false;	/* true when user is done */
	byte *msg;				/* message pointer */
	char *s;				/* file name in the command line */
	int junk;				/* not used */

	cl_init(switches, nswitches, options, noptions, argv, argc);
	if (cl_switch("-help")) instruct();

	/* get input file name: */
	filename[0] = '\0';		/* null string default */
	if ((s = cl_arg(1)) != NULL) strcpy(filename, s);

	while (!done) {
		if (askbool("Do you want instructions", false))
			instruct();
		fp = fileopen(filename, "dx7", "w", "file name");
		if (!midi_buffer(midibuff, BUFF_SIZE))
			printf("midi_buffer error\n");
		musicinit();
		exclusive(true);	/* tell midi drivers to allow exclusive data */

		while (getkey(false, &junk) != -1)
			/* clear out buffer */;

		printf("\nReady for your data. Type space when you are done...\n");
		done = false;
		while (!done) {
			/* loop to allow interrupt handler to collect data */
			done = (stop() != CONTINUE);
			(void) gettime();	/* gettime checks for aborts */
		}

		musicterm();
		msg = midibuff;
		if (xbufftail == 0) {
			printf("No data!  Please start again.\n");
		} else {
			while (output(fp, &msg, midibuff + xbufftail))
				 /* write messages */;
		}
		fclose(fp);
		printf("Done\n");
		done = !askbool("Do you want to send another file", false);
		filename[0] = '\0';		/* force prompt for new file name */
	}
}

/****************************************************************************
*					output
* Inputs:
*	FILE * fp: the file to write
*	byte ** msg: pointer to the message to write
*	byte * msg_tail: points to byte after last byte of recorded data
* Returns:
*	boolean: true if there is more data to output
*	*(byte **msg): is advanced to the byte after the last byte of the message
* Effect:
*	writes data file using recorded data
* Assumes:
*	fp is opened for writing
*	msg_tail > *msg
****************************************************************************/

private boolean output(fp, msg, msg_tail)
	FILE *fp;
	byte **msg;
	byte *msg_tail;
{
	byte *this_msg = *msg;	/* local message pointer */
	int datalen;			/* number of bytes in message */
	int cksum;				/* check sum of message */
	int n;					/* counts bytes printed on one line */

	excldesc(this_msg);
	this_msg = *msg;
	datalen = (this_msg[4] << 7) + this_msg[5];

	for (cksum = 0, n = 6; n < datalen + 6; n++) {
		cksum = (cksum + this_msg[n]) & 0x7f;
	}
	if ((cksum + this_msg[datalen + 6]) & 0x7f != 0) {
		fprintf(stderr, "Data has been garbled -- please start over.\n");
		return false;
	}

	n = 0;
	do {
		/* always print first byte, */
		/* then terminate after printing MIDI_EOX */
		fprintf(fp, "%x ", *this_msg);
		if (n >= 15) {	/* print 16 bytes per line */
			n = 0;
			fprintf(fp, "\n");
		} else n++;
		this_msg++;
	} while ((this_msg < msg_tail) && (*(this_msg - 1) != MIDI_EOX));

	fprintf(fp, "\n\n");
	*msg = this_msg;
	return this_msg < msg_tail;
}

